# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/integrations.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/integrations.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1agrid/v1/integrations.proto\x12\x07grid.v1\"\x86\x01\n\x05Token\x12\x14\n\x05token\x18\x01 \x01(\tR\x05token\x12\x32\n\x08provider\x18\x02 \x01(\x0e\x32\x16.grid.v1.TokenProviderR\x08provider\x12\x14\n\x05scope\x18\x03 \x01(\tR\x05scope\x12\x1d\n\nscope_type\x18\x04 \x01(\tR\tscopeType\"0\n\x15GetGitHubTokenRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\">\n\x16GetGitHubTokenResponse\x12$\n\x05token\x18\x01 \x01(\x0b\x32\x0e.grid.v1.TokenR\x05token\"V\n\x15SetGitHubTokenRequest\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12$\n\x05token\x18\x02 \x01(\x0b\x32\x0e.grid.v1.TokenR\x05token\"\x18\n\x16SetGitHubTokenResponse*J\n\rTokenProvider\x12\x1e\n\x1aTOKEN_PROVIDER_UNSPECIFIED\x10\x00\x12\x19\n\x15TOKEN_PROVIDER_GITHUB\x10\x01\x32\xba\x01\n\x12IntegrationService\x12Q\n\x0eGetGitHubToken\x12\x1e.grid.v1.GetGitHubTokenRequest\x1a\x1f.grid.v1.GetGitHubTokenResponse\x12Q\n\x0eSetGitHubToken\x12\x1e.grid.v1.SetGitHubTokenRequest\x1a\x1f.grid.v1.SetGitHubTokenResponseB2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
)

_TOKENPROVIDER = _descriptor.EnumDescriptor(
  name='TokenProvider',
  full_name='grid.v1.TokenProvider',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TOKEN_PROVIDER_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TOKEN_PROVIDER_GITHUB', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=404,
  serialized_end=478,
)
_sym_db.RegisterEnumDescriptor(_TOKENPROVIDER)

TokenProvider = enum_type_wrapper.EnumTypeWrapper(_TOKENPROVIDER)
TOKEN_PROVIDER_UNSPECIFIED = 0
TOKEN_PROVIDER_GITHUB = 1



_TOKEN = _descriptor.Descriptor(
  name='Token',
  full_name='grid.v1.Token',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='grid.v1.Token.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='token', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='provider', full_name='grid.v1.Token.provider', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='provider', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scope', full_name='grid.v1.Token.scope', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scope', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scope_type', full_name='grid.v1.Token.scope_type', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='scopeType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=40,
  serialized_end=174,
)


_GETGITHUBTOKENREQUEST = _descriptor.Descriptor(
  name='GetGitHubTokenRequest',
  full_name='grid.v1.GetGitHubTokenRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_id', full_name='grid.v1.GetGitHubTokenRequest.user_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=176,
  serialized_end=224,
)


_GETGITHUBTOKENRESPONSE = _descriptor.Descriptor(
  name='GetGitHubTokenResponse',
  full_name='grid.v1.GetGitHubTokenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='grid.v1.GetGitHubTokenResponse.token', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='token', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=226,
  serialized_end=288,
)


_SETGITHUBTOKENREQUEST = _descriptor.Descriptor(
  name='SetGitHubTokenRequest',
  full_name='grid.v1.SetGitHubTokenRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_id', full_name='grid.v1.SetGitHubTokenRequest.user_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='token', full_name='grid.v1.SetGitHubTokenRequest.token', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='token', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=290,
  serialized_end=376,
)


_SETGITHUBTOKENRESPONSE = _descriptor.Descriptor(
  name='SetGitHubTokenResponse',
  full_name='grid.v1.SetGitHubTokenResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=378,
  serialized_end=402,
)

_TOKEN.fields_by_name['provider'].enum_type = _TOKENPROVIDER
_GETGITHUBTOKENRESPONSE.fields_by_name['token'].message_type = _TOKEN
_SETGITHUBTOKENREQUEST.fields_by_name['token'].message_type = _TOKEN
DESCRIPTOR.message_types_by_name['Token'] = _TOKEN
DESCRIPTOR.message_types_by_name['GetGitHubTokenRequest'] = _GETGITHUBTOKENREQUEST
DESCRIPTOR.message_types_by_name['GetGitHubTokenResponse'] = _GETGITHUBTOKENRESPONSE
DESCRIPTOR.message_types_by_name['SetGitHubTokenRequest'] = _SETGITHUBTOKENREQUEST
DESCRIPTOR.message_types_by_name['SetGitHubTokenResponse'] = _SETGITHUBTOKENRESPONSE
DESCRIPTOR.enum_types_by_name['TokenProvider'] = _TOKENPROVIDER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Token = _reflection.GeneratedProtocolMessageType('Token', (_message.Message,), {
  'DESCRIPTOR' : _TOKEN,
  '__module__' : 'grid.v1.integrations_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.Token)
  })
_sym_db.RegisterMessage(Token)

GetGitHubTokenRequest = _reflection.GeneratedProtocolMessageType('GetGitHubTokenRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETGITHUBTOKENREQUEST,
  '__module__' : 'grid.v1.integrations_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.GetGitHubTokenRequest)
  })
_sym_db.RegisterMessage(GetGitHubTokenRequest)

GetGitHubTokenResponse = _reflection.GeneratedProtocolMessageType('GetGitHubTokenResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGITHUBTOKENRESPONSE,
  '__module__' : 'grid.v1.integrations_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.GetGitHubTokenResponse)
  })
_sym_db.RegisterMessage(GetGitHubTokenResponse)

SetGitHubTokenRequest = _reflection.GeneratedProtocolMessageType('SetGitHubTokenRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETGITHUBTOKENREQUEST,
  '__module__' : 'grid.v1.integrations_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.SetGitHubTokenRequest)
  })
_sym_db.RegisterMessage(SetGitHubTokenRequest)

SetGitHubTokenResponse = _reflection.GeneratedProtocolMessageType('SetGitHubTokenResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETGITHUBTOKENRESPONSE,
  '__module__' : 'grid.v1.integrations_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.SetGitHubTokenResponse)
  })
_sym_db.RegisterMessage(SetGitHubTokenResponse)


DESCRIPTOR._options = None

_INTEGRATIONSERVICE = _descriptor.ServiceDescriptor(
  name='IntegrationService',
  full_name='grid.v1.IntegrationService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=481,
  serialized_end=667,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetGitHubToken',
    full_name='grid.v1.IntegrationService.GetGitHubToken',
    index=0,
    containing_service=None,
    input_type=_GETGITHUBTOKENREQUEST,
    output_type=_GETGITHUBTOKENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetGitHubToken',
    full_name='grid.v1.IntegrationService.SetGitHubToken',
    index=1,
    containing_service=None,
    input_type=_SETGITHUBTOKENREQUEST,
    output_type=_SETGITHUBTOKENRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_INTEGRATIONSERVICE)

DESCRIPTOR.services_by_name['IntegrationService'] = _INTEGRATIONSERVICE

# @@protoc_insertion_point(module_scope)



from copy import copy
from pathlib import Path
from typing import Set


def generate(src: Path) -> Set[str]:
    """
    Generates list of glob strings to ignore. These need to be parsed to identify
    which files are actually ignored.

    Parameters
    ----------
    src: Path
        Path to location where to generate ignore list from.

    Returns
    -------
    Set[str]
        Glob strings to parse
    """
    # fallback files in case we can't find .gridignore
    fallback_ignore_files = {".dockerignore", ".gitignore"}

    # never include these paths in the package
    excluded_paths = {".git"}

    # if it is a file, then just read from it and return lines
    if src.is_file():
        return _read_and_filter_gridignore(src)

    # ignores all paths from excluded paths by default
    ignore_globs = {f"{p}/" for p in excluded_paths}

    # look first for `.gridignore` files
    for path in src.glob('**/*'):
        if path.name in excluded_paths:
            continue
        if path.is_file():
            if path.name == ".gridignore":
                filtered = _read_and_filter_gridignore(path)
                relative_dir = path.relative_to(src).parents[0]
                relative_globs = [str(relative_dir / glob) for glob in filtered]
                ignore_globs.update(relative_globs)

    # if found .gridignore, return it
    if len(ignore_globs) > len(excluded_paths):
        return ignore_globs

    # if not found, look everything else -- combine all fallback_ignore_files into one
    for path in src.glob('**/*'):
        if path.name in excluded_paths:
            continue
        if path.is_file():
            if path.name in fallback_ignore_files:
                filtered = _read_and_filter_gridignore(path)
                relative_dir = path.relative_to(src).parents[0]
                relative_globs = [str(relative_dir / glob) for glob in filtered]
                ignore_globs.update(relative_globs)

    return ignore_globs


def _read_and_filter_gridignore(path: Path) -> Set[str]:
    """
    Reads ignore file and filter and empty lines. This will also remove patterns that start with a `/`.
    That's done to allow `glob` to simulate the behavior done by `git` where it interprets that as a
    root path.

    Parameters
    ----------
    path: Path
        Path to .gridignore file or equivalent.

    Returns
    -------
    Set[str]
        Set of unique lines.
    """
    raw_lines = [ln.strip() for ln in path.open().readlines()]

    # creates a set that removes empty lines and comments
    lines = {ln for ln in raw_lines if ln != "" and ln is not None and not ln.startswith("#")}

    # removes first `/` character
    _lines = copy(lines)
    for pattern in _lines:
        if pattern.startswith("/"):
            lines.remove(pattern)
            lines.add(pattern[1:])

    return lines



from __future__ import annotations

import typing
from typing import List

from grid.cli.core.base import GridObject
from grid.cli.exceptions import APIError

if typing.TYPE_CHECKING:
    from grid.cli.client import Grid


class Team(GridObject):
    """
    Team object in Grid
    """
    def refresh(self):
        pass

    @classmethod
    def get_all(cls) -> List['Team']:
        client: Grid = cls().client

        query = """
            query GetUserTeams {
                getUserTeams {
                    success
                    message
                    teams {
                        id
                        name
                        createdAt
                        role
                        members {
                            id
                            username
                            firstName
                            lastName
                        }
                    }
                }
            }
        """
        result = client.query(query)
        if not result['getUserTeams'] or not result['getUserTeams']['success']:
            raise APIError(result['getUserTeams']["message"])
        teams = []
        for detail in result['getUserTeams']['teams']:
            team = cls()
            team.data = detail
            teams.append(team)
        return teams



from typing import List

import click

from grid.cli import rich_click
from grid.cli.client import Grid
from grid.openapi.api import cluster_service_api
from grid.sdk.client import create_swagger_client


@rich_click.group()
def delete() -> None:
    """Allows you to delete grid resources."""
    pass


def doublecheck(item: str):
    warning_str = click.style('WARNING!', fg='red')
    message = f"""

    {warning_str}

    Your are about to delete the {item}.
    This will delete all the associated artifacts, logs, and metadata.

    Are you sure you want to do this?

   """
    click.confirm(message, abort=True)


@delete.command()
@rich_click.argument('experiment_ids', type=str, required=True, nargs=-1, help='Experiment IDs to delete.')
def experiment(experiment_ids: List[str]):
    """Delete some set of EXPERIMENT_IDS from grid.

    This process is immediate and irriversable, deletion permenantly removes not only
    the record of the experiment, but all associated artifacts, metrics, logs, etc.
    """
    doublecheck(experiment_ids)
    client = Grid()
    for experiment in experiment_ids:
        client.delete(experiment_name=experiment)


@delete.command()
@rich_click.argument('run_ids', type=str, required=True, nargs=-1, help='Run IDs to delete.')
def run(run_ids: List[str]):
    """Delete some set of RUN_IDS from grid.

    Deleting a run also deletes all experiments contained within the run.

    This process is immediate and irriversable, deletion permenantly removes not only
    the record of the run, but all associated experiments, artifacts, metrics, logs, etc.
    """
    doublecheck(run_ids)
    client = Grid()
    for run in run_ids:
        client.delete(run_name=run)


@delete.command()
@rich_click.argument('cluster', type=str, help='Cluster id to delete.')
def cluster(cluster: str):
    """Delete CLUSTER and all associated AWS resources.

    Deleting a run also deletes all Runs and Experiments which were started
    on the cluster. deletion permanently removes not only the record of all
    runs on a cluster, but all associated experiments, artifacts, metrics, logs, etc.

    This process may take a few minutes to complete, but once started is irriversable.
    Deletion permanently removes not only cluster from being managed by grid, but tears
    down every resource grid managed (for that cluster id) in the host cloud. All object
    stores, container registries, logs, compute nodes, volumes, etc. are deleted and
    cannot be recovered.
    """
    api_client = create_swagger_client()
    cluster_api = cluster_service_api.ClusterServiceApi(api_client)
    cluster_api.cluster_service_delete_cluster(id=cluster)
    click.echo("Cluster deletion triggered successfully")



import os

import grid.cli.globals as env
from grid.cli.utilities import get_graphql_url
from grid.metadata import __version__
from grid.openapi import ApiClient, Configuration
from grid.sdk.auth import Credentials
from grid.sdk.client.grid_gql import gql_client


def create_gql_client(*, websocket: bool = False):
    # since we are manipulating the GRID_URL on the fly, we always need to check the env variable
    url = get_graphql_url(os.environ.get("GRID_URL") or env.DEFAULT_GRID_URL)
    creds = Credentials.from_locale()
    res = gql_client(url=url, creds=creds, websocket=websocket)
    return res


def create_swagger_client():
    """
    Create the swagger client using the autogenerated code
    """
    url = os.environ.get("GRID_URL") or env.DEFAULT_GRID_URL.rstrip("/graphql")
    creds = Credentials.from_locale()
    configuration = Configuration()
    configuration.host = url
    configuration.username = creds.user_id
    configuration.password = creds.api_key
    # TODO Increase the pool_thread (and make it configurable) and use a singleton
    api_client = ApiClient(
        configuration, header_name="Authorization", header_value=configuration.get_basic_auth_token()
    )
    api_client.user_agent = f'Grid-CLI-{__version__}'
    return api_client



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class V1RunStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cost': 'float',
        'deleted_timestamp': 'datetime',
        'experiment_counts': 'V1ExperimentCounts',
        'experiment_ids': 'list[str]',
        'finish_timestamp': 'datetime',
        'hourly_cost': 'float',
        'message': 'str',
        'phase': 'V1RunState',
        'start_timestamp': 'datetime'
    }

    attribute_map = {
        'cost': 'cost',
        'deleted_timestamp': 'deletedTimestamp',
        'experiment_counts': 'experimentCounts',
        'experiment_ids': 'experimentIds',
        'finish_timestamp': 'finishTimestamp',
        'hourly_cost': 'hourlyCost',
        'message': 'message',
        'phase': 'phase',
        'start_timestamp': 'startTimestamp'
    }

    def __init__(self, cost=None, deleted_timestamp=None, experiment_counts=None, experiment_ids=None, finish_timestamp=None, hourly_cost=None, message=None, phase=None, start_timestamp=None, _configuration=None):  # noqa: E501
        """V1RunStatus - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._cost = None
        self._deleted_timestamp = None
        self._experiment_counts = None
        self._experiment_ids = None
        self._finish_timestamp = None
        self._hourly_cost = None
        self._message = None
        self._phase = None
        self._start_timestamp = None
        self.discriminator = None

        if cost is not None:
            self.cost = cost
        if deleted_timestamp is not None:
            self.deleted_timestamp = deleted_timestamp
        if experiment_counts is not None:
            self.experiment_counts = experiment_counts
        if experiment_ids is not None:
            self.experiment_ids = experiment_ids
        if finish_timestamp is not None:
            self.finish_timestamp = finish_timestamp
        if hourly_cost is not None:
            self.hourly_cost = hourly_cost
        if message is not None:
            self.message = message
        if phase is not None:
            self.phase = phase
        if start_timestamp is not None:
            self.start_timestamp = start_timestamp

    @property
    def cost(self):
        """Gets the cost of this V1RunStatus.  # noqa: E501


        :return: The cost of this V1RunStatus.  # noqa: E501
        :rtype: float
        """
        return self._cost

    @cost.setter
    def cost(self, cost):
        """Sets the cost of this V1RunStatus.


        :param cost: The cost of this V1RunStatus.  # noqa: E501
        :type: float
        """

        self._cost = cost

    @property
    def deleted_timestamp(self):
        """Gets the deleted_timestamp of this V1RunStatus.  # noqa: E501


        :return: The deleted_timestamp of this V1RunStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._deleted_timestamp

    @deleted_timestamp.setter
    def deleted_timestamp(self, deleted_timestamp):
        """Sets the deleted_timestamp of this V1RunStatus.


        :param deleted_timestamp: The deleted_timestamp of this V1RunStatus.  # noqa: E501
        :type: datetime
        """

        self._deleted_timestamp = deleted_timestamp

    @property
    def experiment_counts(self):
        """Gets the experiment_counts of this V1RunStatus.  # noqa: E501


        :return: The experiment_counts of this V1RunStatus.  # noqa: E501
        :rtype: V1ExperimentCounts
        """
        return self._experiment_counts

    @experiment_counts.setter
    def experiment_counts(self, experiment_counts):
        """Sets the experiment_counts of this V1RunStatus.


        :param experiment_counts: The experiment_counts of this V1RunStatus.  # noqa: E501
        :type: V1ExperimentCounts
        """

        self._experiment_counts = experiment_counts

    @property
    def experiment_ids(self):
        """Gets the experiment_ids of this V1RunStatus.  # noqa: E501


        :return: The experiment_ids of this V1RunStatus.  # noqa: E501
        :rtype: list[str]
        """
        return self._experiment_ids

    @experiment_ids.setter
    def experiment_ids(self, experiment_ids):
        """Sets the experiment_ids of this V1RunStatus.


        :param experiment_ids: The experiment_ids of this V1RunStatus.  # noqa: E501
        :type: list[str]
        """

        self._experiment_ids = experiment_ids

    @property
    def finish_timestamp(self):
        """Gets the finish_timestamp of this V1RunStatus.  # noqa: E501


        :return: The finish_timestamp of this V1RunStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._finish_timestamp

    @finish_timestamp.setter
    def finish_timestamp(self, finish_timestamp):
        """Sets the finish_timestamp of this V1RunStatus.


        :param finish_timestamp: The finish_timestamp of this V1RunStatus.  # noqa: E501
        :type: datetime
        """

        self._finish_timestamp = finish_timestamp

    @property
    def hourly_cost(self):
        """Gets the hourly_cost of this V1RunStatus.  # noqa: E501


        :return: The hourly_cost of this V1RunStatus.  # noqa: E501
        :rtype: float
        """
        return self._hourly_cost

    @hourly_cost.setter
    def hourly_cost(self, hourly_cost):
        """Sets the hourly_cost of this V1RunStatus.


        :param hourly_cost: The hourly_cost of this V1RunStatus.  # noqa: E501
        :type: float
        """

        self._hourly_cost = hourly_cost

    @property
    def message(self):
        """Gets the message of this V1RunStatus.  # noqa: E501


        :return: The message of this V1RunStatus.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this V1RunStatus.


        :param message: The message of this V1RunStatus.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def phase(self):
        """Gets the phase of this V1RunStatus.  # noqa: E501


        :return: The phase of this V1RunStatus.  # noqa: E501
        :rtype: V1RunState
        """
        return self._phase

    @phase.setter
    def phase(self, phase):
        """Sets the phase of this V1RunStatus.


        :param phase: The phase of this V1RunStatus.  # noqa: E501
        :type: V1RunState
        """

        self._phase = phase

    @property
    def start_timestamp(self):
        """Gets the start_timestamp of this V1RunStatus.  # noqa: E501


        :return: The start_timestamp of this V1RunStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._start_timestamp

    @start_timestamp.setter
    def start_timestamp(self, start_timestamp):
        """Sets the start_timestamp of this V1RunStatus.


        :param start_timestamp: The start_timestamp of this V1RunStatus.  # noqa: E501
        :type: datetime
        """

        self._start_timestamp = start_timestamp

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1RunStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1RunStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1RunStatus):
            return True

        return self.to_dict() != other.to_dict()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from grid.protos.grid.v1 import datastore_service_pb2 as grid_dot_v1_dot_datastore__service__pb2


class DatastoreServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateDatastore = channel.unary_unary(
        '/grid.v1.DatastoreService/CreateDatastore',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.CreateDatastoreRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.CreateDatastoreResponse.FromString,
        )
    self.GetDatastore = channel.unary_unary(
        '/grid.v1.DatastoreService/GetDatastore',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.GetDatastoreRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.GetDatastoreResponse.FromString,
        )
    self.ListDatastores = channel.unary_unary(
        '/grid.v1.DatastoreService/ListDatastores',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.ListDatastoresRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.ListDatastoresResponse.FromString,
        )
    self.UpdateDatastore = channel.unary_unary(
        '/grid.v1.DatastoreService/UpdateDatastore',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreResponse.FromString,
        )
    self.UpdateDatastoreStatus = channel.unary_unary(
        '/grid.v1.DatastoreService/UpdateDatastoreStatus',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreStatusRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreStatusResponse.FromString,
        )
    self.DeleteDatastore = channel.unary_unary(
        '/grid.v1.DatastoreService/DeleteDatastore',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.DeleteDatastoreRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.DeleteDatastoreResponse.FromString,
        )
    self.WatchDatastores = channel.unary_stream(
        '/grid.v1.DatastoreService/WatchDatastores',
        request_serializer=grid_dot_v1_dot_datastore__service__pb2.WatchDatastoresRequest.SerializeToString,
        response_deserializer=grid_dot_v1_dot_datastore__service__pb2.WatchDatastoresResponse.FromString,
        )


class DatastoreServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def CreateDatastore(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetDatastore(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListDatastores(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateDatastore(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateDatastoreStatus(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteDatastore(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def WatchDatastores(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_DatastoreServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateDatastore': grpc.unary_unary_rpc_method_handler(
          servicer.CreateDatastore,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.CreateDatastoreRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.CreateDatastoreResponse.SerializeToString,
      ),
      'GetDatastore': grpc.unary_unary_rpc_method_handler(
          servicer.GetDatastore,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.GetDatastoreRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.GetDatastoreResponse.SerializeToString,
      ),
      'ListDatastores': grpc.unary_unary_rpc_method_handler(
          servicer.ListDatastores,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.ListDatastoresRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.ListDatastoresResponse.SerializeToString,
      ),
      'UpdateDatastore': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateDatastore,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreResponse.SerializeToString,
      ),
      'UpdateDatastoreStatus': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateDatastoreStatus,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreStatusRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.UpdateDatastoreStatusResponse.SerializeToString,
      ),
      'DeleteDatastore': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteDatastore,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.DeleteDatastoreRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.DeleteDatastoreResponse.SerializeToString,
      ),
      'WatchDatastores': grpc.unary_stream_rpc_method_handler(
          servicer.WatchDatastores,
          request_deserializer=grid_dot_v1_dot_datastore__service__pb2.WatchDatastoresRequest.FromString,
          response_serializer=grid_dot_v1_dot_datastore__service__pb2.WatchDatastoresResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'grid.v1.DatastoreService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



from typing import Optional

import click

from grid.cli import rich_click
from grid.cli.client import Grid
from grid.sdk._gql.queries import get_user
from grid.sdk.datastore import parse_name_from_source
from grid.sdk.utils.datastore_uploader import clear_cache

WARNING_STR = click.style('WARNING', fg='yellow')


@click.group(invoke_without_command=True)
@click.pass_context
@click.option(
    '--global',
    'is_global',
    type=bool,
    is_flag=True,
    help='Fetch sessions from everyone in the team when flag is passed'
)
def datastore(ctx, is_global: bool) -> None:
    """Manages Datastore workflows."""
    client = Grid()

    if ctx.invoked_subcommand is None:
        client.list_datastores(is_global=is_global)
    elif is_global:
        click.echo(f"{WARNING_STR}: --global flag doesn't have any effect when invoked with a subcommand")


@datastore.command()
@rich_click.argument('session_name', nargs=1, help="The name of the session.")
@click.pass_context
def resume(ctx, session_name: str):
    """Resume uploading a datastore. SESSION_NAME identifies the datastore upload session to resume.

    The SESSION_NAME argument is displayed which starting (or resuming) an upload.
    """
    client = Grid()
    if session_name == "list":
        client.list_resumable_datastore_sessions()
        return

    client.resume_datastore_session(session_name)


@datastore.command(cls=rich_click.deprecate_grid_options())
@click.option(
    '--source',
    required=True,
    help=(
        "Source to create datastore from. This could either be a local "
        "directory (e.g: /opt/local_folder) or a remote HTTP URL pointing "
        "to a TAR file (e.g: http://some_domain/data.tar.gz)."
    )
)
@click.option('--name', type=str, required=False, help='Name of the datastore')
@click.option(
    '--compression',
    type=bool,
    required=False,
    help='Compresses datastores with GZIP when flag is passed.',
    default=False,
    is_flag=True
)
@click.option(
    '--cluster',
    type=str,
    required=False,
    help='cluster id to create the datastore on. (Bring Your Own Cloud Customers Only).'
)
@click.pass_context
def create(ctx, source: str, name: str, compression: bool = False, cluster: Optional[str] = None) -> None:
    """Creates a datastore and begins the process of uploading files.
    The upload session is referenced by the SESSION_NAME. this SESSION_NAME
    must be used to resume the upload if it is interupted.
    """
    client = Grid()
    user_data = get_user()
    client.check_is_blocked(user_data=user_data)
    if not name:
        name = parse_name_from_source(source)
    client.upload_datastore(source=source, name=name, compression=compression, cluster=cluster)


@datastore.command()
@click.pass_context
def list(ctx) -> None:
    """Lists datastores."""
    client = Grid()
    client.list_datastores()


@datastore.command()
@click.pass_context
def clearcache(ctx) -> None:
    """Clears datastore cache which is saved on the local machine when uploading a datastore to grid.

    This removes all the cached files from the local machine, meaning that resuming an incomplete
    upload is not possible after running this command.
    """
    clear_cache()
    click.echo("Datastore cache cleared")


@datastore.command(cls=rich_click.deprecate_grid_options())
@click.option('--name', type=str, required=True, help='Name of the datastore')
@click.option('--version', type=int, required=True, help='Version of the datastore')
@click.option(
    '--cluster',
    type=str,
    required=False,
    help='cluster id to delete the datastore from. (Bring Your Own Cloud Customers Only).'
)
@click.pass_context
def delete(ctx, name: str, version: int, cluster: Optional[str] = None) -> None:
    """Deletes a datastore with the given name and version tag.

    For bring-your-own-cloud customers, the cluster id of the associated
    resource is required as well.
    """
    client = Grid()
    client.delete_datastore(name=name, version=version, cluster=cluster)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/datastore_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grid.protos.grid.v1 import event_pb2 as grid_dot_v1_dot_event__pb2
from grid.protos.grid.v1 import datastore_pb2 as grid_dot_v1_dot_datastore__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/datastore_service.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1fgrid/v1/datastore_service.proto\x12\x07grid.v1\x1a\x13grid/v1/event.proto\x1a\x17grid/v1/datastore.proto\"J\n\x16\x43reateDatastoreRequest\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"K\n\x17\x43reateDatastoreResponse\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"%\n\x13GetDatastoreRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"H\n\x14GetDatastoreResponse\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"\xd7\x01\n\x15ListDatastoresRequest\x12%\n\x0e\x63ontinue_token\x18\x01 \x01(\tR\rcontinueToken\x12)\n\x10resource_version\x18\x02 \x01(\tR\x0fresourceVersion\x12\x14\n\x05limit\x18\x03 \x01(\x03R\x05limit\x12\x19\n\x08user_ids\x18\x04 \x03(\tR\x07userIds\x12\x1c\n\tavailable\x18\x05 \x01(\x08R\tavailable\x12\x1d\n\ncluster_id\x18\x06 \x01(\tR\tclusterId\"\x9c\x01\n\x16ListDatastoresResponse\x12\x30\n\tdatastore\x18\x01 \x03(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\x12%\n\x0e\x63ontinue_token\x18\x02 \x01(\tR\rcontinueToken\x12)\n\x10resource_version\x18\x03 \x01(\tR\x0fresourceVersion\"J\n\x16UpdateDatastoreRequest\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"K\n\x17UpdateDatastoreResponse\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"P\n\x1cUpdateDatastoreStatusRequest\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"Q\n\x1dUpdateDatastoreStatusResponse\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"J\n\x16\x44\x65leteDatastoreRequest\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"K\n\x17\x44\x65leteDatastoreResponse\x12\x30\n\tdatastore\x18\x01 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore\"\xb1\x01\n\x16WatchDatastoresRequest\x12)\n\x10resource_version\x18\x01 \x01(\tR\x0fresourceVersion\x12\x14\n\x05limit\x18\x02 \x01(\x03R\x05limit\x12\x19\n\x08user_ids\x18\x04 \x03(\tR\x07userIds\x12\x1c\n\tavailable\x18\x05 \x01(\x08R\tavailable\x12\x1d\n\ncluster_id\x18\x06 \x01(\tR\tclusterId\"\x9e\x01\n\x17WatchDatastoresResponse\x12)\n\x10resource_version\x18\x01 \x01(\tR\x0fresourceVersion\x12&\n\x04type\x18\x02 \x01(\x0e\x32\x12.grid.v1.EventTypeR\x04type\x12\x30\n\tdatastore\x18\x03 \x01(\x0b\x32\x12.grid.v1.DatastoreR\tdatastore2\xfe\x04\n\x10\x44\x61tastoreService\x12V\n\x0f\x43reateDatastore\x12\x1f.grid.v1.CreateDatastoreRequest\x1a .grid.v1.CreateDatastoreResponse\"\x00\x12M\n\x0cGetDatastore\x12\x1c.grid.v1.GetDatastoreRequest\x1a\x1d.grid.v1.GetDatastoreResponse\"\x00\x12S\n\x0eListDatastores\x12\x1e.grid.v1.ListDatastoresRequest\x1a\x1f.grid.v1.ListDatastoresResponse\"\x00\x12V\n\x0fUpdateDatastore\x12\x1f.grid.v1.UpdateDatastoreRequest\x1a .grid.v1.UpdateDatastoreResponse\"\x00\x12\x66\n\x15UpdateDatastoreStatus\x12%.grid.v1.UpdateDatastoreStatusRequest\x1a&.grid.v1.UpdateDatastoreStatusResponse\x12V\n\x0f\x44\x65leteDatastore\x12\x1f.grid.v1.DeleteDatastoreRequest\x1a .grid.v1.DeleteDatastoreResponse\"\x00\x12V\n\x0fWatchDatastores\x12\x1f.grid.v1.WatchDatastoresRequest\x1a .grid.v1.WatchDatastoresResponse0\x01\x42\x32Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[grid_dot_v1_dot_event__pb2.DESCRIPTOR,grid_dot_v1_dot_datastore__pb2.DESCRIPTOR,])




_CREATEDATASTOREREQUEST = _descriptor.Descriptor(
  name='CreateDatastoreRequest',
  full_name='grid.v1.CreateDatastoreRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.CreateDatastoreRequest.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=90,
  serialized_end=164,
)


_CREATEDATASTORERESPONSE = _descriptor.Descriptor(
  name='CreateDatastoreResponse',
  full_name='grid.v1.CreateDatastoreResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.CreateDatastoreResponse.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=166,
  serialized_end=241,
)


_GETDATASTOREREQUEST = _descriptor.Descriptor(
  name='GetDatastoreRequest',
  full_name='grid.v1.GetDatastoreRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='grid.v1.GetDatastoreRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=243,
  serialized_end=280,
)


_GETDATASTORERESPONSE = _descriptor.Descriptor(
  name='GetDatastoreResponse',
  full_name='grid.v1.GetDatastoreResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.GetDatastoreResponse.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=282,
  serialized_end=354,
)


_LISTDATASTORESREQUEST = _descriptor.Descriptor(
  name='ListDatastoresRequest',
  full_name='grid.v1.ListDatastoresRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='continue_token', full_name='grid.v1.ListDatastoresRequest.continue_token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='continueToken', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.ListDatastoresRequest.resource_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='limit', full_name='grid.v1.ListDatastoresRequest.limit', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='limit', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_ids', full_name='grid.v1.ListDatastoresRequest.user_ids', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='available', full_name='grid.v1.ListDatastoresRequest.available', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='available', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.ListDatastoresRequest.cluster_id', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=357,
  serialized_end=572,
)


_LISTDATASTORESRESPONSE = _descriptor.Descriptor(
  name='ListDatastoresResponse',
  full_name='grid.v1.ListDatastoresResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.ListDatastoresResponse.datastore', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='continue_token', full_name='grid.v1.ListDatastoresResponse.continue_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='continueToken', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.ListDatastoresResponse.resource_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=575,
  serialized_end=731,
)


_UPDATEDATASTOREREQUEST = _descriptor.Descriptor(
  name='UpdateDatastoreRequest',
  full_name='grid.v1.UpdateDatastoreRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.UpdateDatastoreRequest.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=733,
  serialized_end=807,
)


_UPDATEDATASTORERESPONSE = _descriptor.Descriptor(
  name='UpdateDatastoreResponse',
  full_name='grid.v1.UpdateDatastoreResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.UpdateDatastoreResponse.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=809,
  serialized_end=884,
)


_UPDATEDATASTORESTATUSREQUEST = _descriptor.Descriptor(
  name='UpdateDatastoreStatusRequest',
  full_name='grid.v1.UpdateDatastoreStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.UpdateDatastoreStatusRequest.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=886,
  serialized_end=966,
)


_UPDATEDATASTORESTATUSRESPONSE = _descriptor.Descriptor(
  name='UpdateDatastoreStatusResponse',
  full_name='grid.v1.UpdateDatastoreStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.UpdateDatastoreStatusResponse.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=968,
  serialized_end=1049,
)


_DELETEDATASTOREREQUEST = _descriptor.Descriptor(
  name='DeleteDatastoreRequest',
  full_name='grid.v1.DeleteDatastoreRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.DeleteDatastoreRequest.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1051,
  serialized_end=1125,
)


_DELETEDATASTORERESPONSE = _descriptor.Descriptor(
  name='DeleteDatastoreResponse',
  full_name='grid.v1.DeleteDatastoreResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.DeleteDatastoreResponse.datastore', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1127,
  serialized_end=1202,
)


_WATCHDATASTORESREQUEST = _descriptor.Descriptor(
  name='WatchDatastoresRequest',
  full_name='grid.v1.WatchDatastoresRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.WatchDatastoresRequest.resource_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='limit', full_name='grid.v1.WatchDatastoresRequest.limit', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='limit', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_ids', full_name='grid.v1.WatchDatastoresRequest.user_ids', index=2,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='available', full_name='grid.v1.WatchDatastoresRequest.available', index=3,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='available', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.WatchDatastoresRequest.cluster_id', index=4,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1205,
  serialized_end=1382,
)


_WATCHDATASTORESRESPONSE = _descriptor.Descriptor(
  name='WatchDatastoresResponse',
  full_name='grid.v1.WatchDatastoresResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.WatchDatastoresResponse.resource_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='grid.v1.WatchDatastoresResponse.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datastore', full_name='grid.v1.WatchDatastoresResponse.datastore', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastore', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1385,
  serialized_end=1543,
)

_CREATEDATASTOREREQUEST.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_CREATEDATASTORERESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_GETDATASTORERESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_LISTDATASTORESRESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_UPDATEDATASTOREREQUEST.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_UPDATEDATASTORERESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_UPDATEDATASTORESTATUSREQUEST.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_UPDATEDATASTORESTATUSRESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_DELETEDATASTOREREQUEST.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_DELETEDATASTORERESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
_WATCHDATASTORESRESPONSE.fields_by_name['type'].enum_type = grid_dot_v1_dot_event__pb2._EVENTTYPE
_WATCHDATASTORESRESPONSE.fields_by_name['datastore'].message_type = grid_dot_v1_dot_datastore__pb2._DATASTORE
DESCRIPTOR.message_types_by_name['CreateDatastoreRequest'] = _CREATEDATASTOREREQUEST
DESCRIPTOR.message_types_by_name['CreateDatastoreResponse'] = _CREATEDATASTORERESPONSE
DESCRIPTOR.message_types_by_name['GetDatastoreRequest'] = _GETDATASTOREREQUEST
DESCRIPTOR.message_types_by_name['GetDatastoreResponse'] = _GETDATASTORERESPONSE
DESCRIPTOR.message_types_by_name['ListDatastoresRequest'] = _LISTDATASTORESREQUEST
DESCRIPTOR.message_types_by_name['ListDatastoresResponse'] = _LISTDATASTORESRESPONSE
DESCRIPTOR.message_types_by_name['UpdateDatastoreRequest'] = _UPDATEDATASTOREREQUEST
DESCRIPTOR.message_types_by_name['UpdateDatastoreResponse'] = _UPDATEDATASTORERESPONSE
DESCRIPTOR.message_types_by_name['UpdateDatastoreStatusRequest'] = _UPDATEDATASTORESTATUSREQUEST
DESCRIPTOR.message_types_by_name['UpdateDatastoreStatusResponse'] = _UPDATEDATASTORESTATUSRESPONSE
DESCRIPTOR.message_types_by_name['DeleteDatastoreRequest'] = _DELETEDATASTOREREQUEST
DESCRIPTOR.message_types_by_name['DeleteDatastoreResponse'] = _DELETEDATASTORERESPONSE
DESCRIPTOR.message_types_by_name['WatchDatastoresRequest'] = _WATCHDATASTORESREQUEST
DESCRIPTOR.message_types_by_name['WatchDatastoresResponse'] = _WATCHDATASTORESRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateDatastoreRequest = _reflection.GeneratedProtocolMessageType('CreateDatastoreRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDATASTOREREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.CreateDatastoreRequest)
  })
_sym_db.RegisterMessage(CreateDatastoreRequest)

CreateDatastoreResponse = _reflection.GeneratedProtocolMessageType('CreateDatastoreResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDATASTORERESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.CreateDatastoreResponse)
  })
_sym_db.RegisterMessage(CreateDatastoreResponse)

GetDatastoreRequest = _reflection.GeneratedProtocolMessageType('GetDatastoreRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDATASTOREREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.GetDatastoreRequest)
  })
_sym_db.RegisterMessage(GetDatastoreRequest)

GetDatastoreResponse = _reflection.GeneratedProtocolMessageType('GetDatastoreResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDATASTORERESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.GetDatastoreResponse)
  })
_sym_db.RegisterMessage(GetDatastoreResponse)

ListDatastoresRequest = _reflection.GeneratedProtocolMessageType('ListDatastoresRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTDATASTORESREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ListDatastoresRequest)
  })
_sym_db.RegisterMessage(ListDatastoresRequest)

ListDatastoresResponse = _reflection.GeneratedProtocolMessageType('ListDatastoresResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTDATASTORESRESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ListDatastoresResponse)
  })
_sym_db.RegisterMessage(ListDatastoresResponse)

UpdateDatastoreRequest = _reflection.GeneratedProtocolMessageType('UpdateDatastoreRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATASTOREREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateDatastoreRequest)
  })
_sym_db.RegisterMessage(UpdateDatastoreRequest)

UpdateDatastoreResponse = _reflection.GeneratedProtocolMessageType('UpdateDatastoreResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATASTORERESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateDatastoreResponse)
  })
_sym_db.RegisterMessage(UpdateDatastoreResponse)

UpdateDatastoreStatusRequest = _reflection.GeneratedProtocolMessageType('UpdateDatastoreStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATASTORESTATUSREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateDatastoreStatusRequest)
  })
_sym_db.RegisterMessage(UpdateDatastoreStatusRequest)

UpdateDatastoreStatusResponse = _reflection.GeneratedProtocolMessageType('UpdateDatastoreStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATASTORESTATUSRESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateDatastoreStatusResponse)
  })
_sym_db.RegisterMessage(UpdateDatastoreStatusResponse)

DeleteDatastoreRequest = _reflection.GeneratedProtocolMessageType('DeleteDatastoreRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDATASTOREREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DeleteDatastoreRequest)
  })
_sym_db.RegisterMessage(DeleteDatastoreRequest)

DeleteDatastoreResponse = _reflection.GeneratedProtocolMessageType('DeleteDatastoreResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDATASTORERESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DeleteDatastoreResponse)
  })
_sym_db.RegisterMessage(DeleteDatastoreResponse)

WatchDatastoresRequest = _reflection.GeneratedProtocolMessageType('WatchDatastoresRequest', (_message.Message,), {
  'DESCRIPTOR' : _WATCHDATASTORESREQUEST,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.WatchDatastoresRequest)
  })
_sym_db.RegisterMessage(WatchDatastoresRequest)

WatchDatastoresResponse = _reflection.GeneratedProtocolMessageType('WatchDatastoresResponse', (_message.Message,), {
  'DESCRIPTOR' : _WATCHDATASTORESRESPONSE,
  '__module__' : 'grid.v1.datastore_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.WatchDatastoresResponse)
  })
_sym_db.RegisterMessage(WatchDatastoresResponse)


DESCRIPTOR._options = None

_DATASTORESERVICE = _descriptor.ServiceDescriptor(
  name='DatastoreService',
  full_name='grid.v1.DatastoreService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1546,
  serialized_end=2184,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateDatastore',
    full_name='grid.v1.DatastoreService.CreateDatastore',
    index=0,
    containing_service=None,
    input_type=_CREATEDATASTOREREQUEST,
    output_type=_CREATEDATASTORERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDatastore',
    full_name='grid.v1.DatastoreService.GetDatastore',
    index=1,
    containing_service=None,
    input_type=_GETDATASTOREREQUEST,
    output_type=_GETDATASTORERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListDatastores',
    full_name='grid.v1.DatastoreService.ListDatastores',
    index=2,
    containing_service=None,
    input_type=_LISTDATASTORESREQUEST,
    output_type=_LISTDATASTORESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateDatastore',
    full_name='grid.v1.DatastoreService.UpdateDatastore',
    index=3,
    containing_service=None,
    input_type=_UPDATEDATASTOREREQUEST,
    output_type=_UPDATEDATASTORERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateDatastoreStatus',
    full_name='grid.v1.DatastoreService.UpdateDatastoreStatus',
    index=4,
    containing_service=None,
    input_type=_UPDATEDATASTORESTATUSREQUEST,
    output_type=_UPDATEDATASTORESTATUSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteDatastore',
    full_name='grid.v1.DatastoreService.DeleteDatastore',
    index=5,
    containing_service=None,
    input_type=_DELETEDATASTOREREQUEST,
    output_type=_DELETEDATASTORERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='WatchDatastores',
    full_name='grid.v1.DatastoreService.WatchDatastores',
    index=6,
    containing_service=None,
    input_type=_WATCHDATASTORESREQUEST,
    output_type=_WATCHDATASTORESRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DATASTORESERVICE)

DESCRIPTOR.services_by_name['DatastoreService'] = _DATASTORESERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/whoami.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grid.protos.grid.v1 import user_pb2 as grid_dot_v1_dot_user__pb2
from grid.protos.grid.v1 import cluster_pb2 as grid_dot_v1_dot_cluster__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/whoami.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14grid/v1/whoami.proto\x12\x07grid.v1\x1a\x12grid/v1/user.proto\x1a\x15grid/v1/cluster.proto\"\x0f\n\rWhoAmIRequest\"\x8a\x01\n\x0eWhoAmIResponse\x12#\n\x04user\x18\x01 \x01(\x0b\x32\r.grid.v1.UserH\x00R\x04user\x12,\n\x07\x63luster\x18\x02 \x01(\x0b\x32\x10.grid.v1.ClusterH\x00R\x07\x63luster\x12\x1c\n\x08internal\x18\x03 \x01(\tH\x00R\x08internalB\x07\n\x05\x61\x63tor2R\n\x15\x41uthenticationService\x12\x39\n\x06WhoAmI\x12\x16.grid.v1.WhoAmIRequest\x1a\x17.grid.v1.WhoAmIResponseB2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[grid_dot_v1_dot_user__pb2.DESCRIPTOR,grid_dot_v1_dot_cluster__pb2.DESCRIPTOR,])




_WHOAMIREQUEST = _descriptor.Descriptor(
  name='WhoAmIRequest',
  full_name='grid.v1.WhoAmIRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=76,
  serialized_end=91,
)


_WHOAMIRESPONSE = _descriptor.Descriptor(
  name='WhoAmIResponse',
  full_name='grid.v1.WhoAmIResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='grid.v1.WhoAmIResponse.user', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='user', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster', full_name='grid.v1.WhoAmIResponse.cluster', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='cluster', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='internal', full_name='grid.v1.WhoAmIResponse.internal', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='internal', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='actor', full_name='grid.v1.WhoAmIResponse.actor',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=94,
  serialized_end=232,
)

_WHOAMIRESPONSE.fields_by_name['user'].message_type = grid_dot_v1_dot_user__pb2._USER
_WHOAMIRESPONSE.fields_by_name['cluster'].message_type = grid_dot_v1_dot_cluster__pb2._CLUSTER
_WHOAMIRESPONSE.oneofs_by_name['actor'].fields.append(
  _WHOAMIRESPONSE.fields_by_name['user'])
_WHOAMIRESPONSE.fields_by_name['user'].containing_oneof = _WHOAMIRESPONSE.oneofs_by_name['actor']
_WHOAMIRESPONSE.oneofs_by_name['actor'].fields.append(
  _WHOAMIRESPONSE.fields_by_name['cluster'])
_WHOAMIRESPONSE.fields_by_name['cluster'].containing_oneof = _WHOAMIRESPONSE.oneofs_by_name['actor']
_WHOAMIRESPONSE.oneofs_by_name['actor'].fields.append(
  _WHOAMIRESPONSE.fields_by_name['internal'])
_WHOAMIRESPONSE.fields_by_name['internal'].containing_oneof = _WHOAMIRESPONSE.oneofs_by_name['actor']
DESCRIPTOR.message_types_by_name['WhoAmIRequest'] = _WHOAMIREQUEST
DESCRIPTOR.message_types_by_name['WhoAmIResponse'] = _WHOAMIRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

WhoAmIRequest = _reflection.GeneratedProtocolMessageType('WhoAmIRequest', (_message.Message,), {
  'DESCRIPTOR' : _WHOAMIREQUEST,
  '__module__' : 'grid.v1.whoami_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.WhoAmIRequest)
  })
_sym_db.RegisterMessage(WhoAmIRequest)

WhoAmIResponse = _reflection.GeneratedProtocolMessageType('WhoAmIResponse', (_message.Message,), {
  'DESCRIPTOR' : _WHOAMIRESPONSE,
  '__module__' : 'grid.v1.whoami_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.WhoAmIResponse)
  })
_sym_db.RegisterMessage(WhoAmIResponse)


DESCRIPTOR._options = None

_AUTHENTICATIONSERVICE = _descriptor.ServiceDescriptor(
  name='AuthenticationService',
  full_name='grid.v1.AuthenticationService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=234,
  serialized_end=316,
  methods=[
  _descriptor.MethodDescriptor(
    name='WhoAmI',
    full_name='grid.v1.AuthenticationService.WhoAmI',
    index=0,
    containing_service=None,
    input_type=_WHOAMIREQUEST,
    output_type=_WHOAMIRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_AUTHENTICATIONSERVICE)

DESCRIPTOR.services_by_name['AuthenticationService'] = _AUTHENTICATIONSERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/cost_controller_service_account.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/cost_controller_service_account.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n-grid/v1/cost_controller_service_account.proto\x12\x07grid.v1\"\x1e\n\x1c\x43ostControllerServiceAccountB2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
)




_COSTCONTROLLERSERVICEACCOUNT = _descriptor.Descriptor(
  name='CostControllerServiceAccount',
  full_name='grid.v1.CostControllerServiceAccount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=58,
  serialized_end=88,
)

DESCRIPTOR.message_types_by_name['CostControllerServiceAccount'] = _COSTCONTROLLERSERVICEACCOUNT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CostControllerServiceAccount = _reflection.GeneratedProtocolMessageType('CostControllerServiceAccount', (_message.Message,), {
  'DESCRIPTOR' : _COSTCONTROLLERSERVICEACCOUNT,
  '__module__' : 'grid.v1.cost_controller_service_account_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.CostControllerServiceAccount)
  })
_sym_db.RegisterMessage(CostControllerServiceAccount)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from datetime import datetime, timedelta
import json
import os
from pathlib import Path
from typing import Union
import uuid

import grpc
import jwt


@dataclass(unsafe_hash=True)
class Credentials:
    user_id: str
    api_key: str

    @classmethod
    def from_locale(cls) -> "Credentials":
        """Instantiates the credentials using implicit locale.

        First use environment variables, otherwise look for credentials stored in file.

        Returns
        -------
        Credentials
            instantiated credentials object.
        """
        # if user has environment variables, use that
        user_id = os.getenv('GRID_USER_ID')
        api_key = os.getenv('GRID_API_KEY')
        if user_id and api_key:
            return cls(user_id=user_id, api_key=api_key)

        # otherwise overwrite look for credentials stored locally as a file
        if os.getenv("CI"):
            p = Path.home() / ".grid" / "credentials.json"
        else:
            p = Path(os.getenv('GRID_CREDENTIAL_PATH', Path.home() / ".grid" / "credentials.json"))

        if not p.exists():
            raise PermissionError('No credentials available. Did you login?')
        with p.open() as f:
            credentials = json.load(f)

        return cls(
            user_id=credentials['UserID'],
            api_key=credentials['APIKey'],
        )

    @classmethod
    def from_file(cls, path: Union[str, Path]) -> "Credentials":
        """Instantiates the credentials using specified file path.

        Parameters
        ----------
        path
            file path to the config yaml or json on disk.

        Returns
        -------
        Credentials
            instantiated credentials object.
        """
        p = Path(path).absolute()
        if not p.exists():
            raise PermissionError('No credentials available. Did you login?')
        with p.open() as f:
            credentials = json.load(f)

        return cls(
            user_id=credentials['UserID'],
            api_key=credentials['APIKey'],
        )


class GrpcAuth(grpc.AuthMetadataPlugin):
    def __init__(self, credentials: "Credentials"):
        self._creds = credentials

    @property
    def _bearer_token(self) -> str:
        if os.getenv("GRID_AUTH_TOKEN"):
            return os.getenv("GRID_AUTH_TOKEN")

        dt_now = datetime.utcnow()
        # TODO(rusenask): Delete this when you migrate everything to nice JWTs
        token = jwt.encode(
            payload={
                "aud": ["grid"],
                "exp": dt_now + timedelta(seconds=30),
                "iat": dt_now,
                "iss": "grid-cli",
                "jti": str(uuid.uuid4()),
                "nbf": dt_now,
                "sub": self._creds.user_id
            },
            key=self._creds.api_key,
            algorithm="HS256"
        )

        return token.decode("ascii")

    def __call__(self, context: "grpc.AuthMetadataContext", callback: "grpc.AuthMetadataPluginCallback") -> None:
        callback((('authorization', f"Bearer {self._bearer_token}"), ), None)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from grid.openapi.api.auth_service_api import AuthServiceApi
from grid.openapi.api.cluster_service_api import ClusterServiceApi
from grid.openapi.api.datastore_service_api import DatastoreServiceApi
from grid.openapi.api.run_service_api import RunServiceApi
from grid.openapi.api.saml_organizations_service_api import SAMLOrganizationsServiceApi
from grid.openapi.api.session_service_api import SessionServiceApi
from grid.openapi.api.tensorboard_service_api import TensorboardServiceApi



import subprocess
from typing import Dict, Optional

import click

from grid.cli import rich_click
from grid.cli.cli.grid_run import _check_run_name_is_valid, _get_instance_types, _resolve_instance_type_nickname
from grid.cli.cli.utilities import read_config_callback, validate_config, validate_disk_size_callback
from grid.cli.client import Grid
import grid.cli.globals as env
from grid.cli.types import ObservableType
from grid.cli.utilities import check_description_isnt_too_long
from grid.sdk.action import validate_and_configure_datastore
from grid.sdk.utils.name_generator import unique_name

WARNING_STR = click.style('WARNING', fg='yellow')


def _generate_default_interactive_config(
    instance_type: str,
    cluster: Optional[str] = None,
    datastore_name: Optional[str] = None,
    datastore_version: Optional[str] = None,
    datastore_mount_dir: Optional[str] = None,
    disk_size: Optional[int] = None,
    use_spot: Optional[bool] = None
):
    """
    Generates a new default config file for user if user hasn't
    submitted one.
    """
    # These configs need to be cleaned-up. We don't really need
    # anything in provider apart from cluster
    provider = {'vendor': "aws", 'credentials': "n/a", 'region': 'us-east-1'}
    if cluster:
        provider['cluster'] = cluster

    _grid_config = {
        'compute': {
            'provider': provider,
            'train': {
                'instance': instance_type,
                'disk_size': disk_size,
                'gpus': 0,  # Required, but ignored.
                'datastore_name': datastore_name,
                'datastore_version': datastore_version,
                'datastore_mount_dir': datastore_mount_dir,
                'use_spot': use_spot or False,
            }
        }
    }

    #  Print debug message if in debug mode.
    if env.DEBUG:
        click.echo('Grid Config used:')
        click.echo(_grid_config)

    return _grid_config


@rich_click.group(invoke_without_command=True)
@click.pass_context
@click.option(
    '--global',
    'is_global',
    type=bool,
    is_flag=True,
    help='Fetch sessions from everyone in the team when flag is passed'
)
def session(ctx, is_global: bool) -> None:
    """Manage Sessions workflows."""
    client = Grid()
    if ctx.invoked_subcommand is None:
        # Get the status of the interactive observables.
        kind = ObservableType.INTERACTIVE
        client.status(kind=kind, follow=False, is_global=is_global)
    elif is_global:
        click.echo(f"{WARNING_STR}: --global flag doesn't have any effect when invoked with a subcommand")


@session.command(cls=rich_click.deprecate_grid_options())
@click.option(
    '--config',
    'config',
    type=click.File('r'),
    required=False,
    callback=read_config_callback,
    help='Path to Grid config YML'
)
@click.option('--name', 'name', type=str, required=False, help='Name for this run', callback=_check_run_name_is_valid)
@click.option(
    '--description',
    'description',
    type=str,
    required=False,
    help='Session description; useful for note-keeping.',
    callback=check_description_isnt_too_long
)
@click.option('--cluster', 'cluster', type=str, required=False, help='Cluster to run on')
@click.option(
    '--instance_type',
    'instance_type',
    type=str,
    default='t2.medium',
    callback=_resolve_instance_type_nickname,
    help='Instance type to start session in.',
    autocompletion=_get_instance_types
)
@click.option(
    '--use_spot',
    'use_spot',
    is_flag=True,
    required=False,
    help='Use spot instance. The spot instances, or preemptive instance can be shut down at will'
)
@click.option(
    '--disk_size',
    'disk_size',
    type=int,
    required=False,
    default=200,
    callback=validate_disk_size_callback,
    help='The disk size in GB to allocate to the session.'
)
@click.option(
    '--datastore_name', 'datastore_name', type=str, required=False, help='Datastore name to be mounted in the session.'
)
@click.option(
    '--datastore_version',
    'datastore_version',
    type=str,
    required=False,
    help='Datastore version to be mounted in the session.'
)
@click.option(
    '--datastore_mount_dir',
    'datastore_mount_dir',
    type=str,
    required=False,
    help='Absolute path to mount Datastore in the session (defaults to ~/[datastore_name]).'
)
def create(
    config: Optional[Dict], name: Optional[str], description: Optional[str], cluster: Optional[str], instance_type: str,
    datastore_name: Optional[str], datastore_version: Optional[str], datastore_mount_dir: Optional[str],
    disk_size: Optional[int], use_spot: Optional[bool]
) -> None:
    """Creates a new interactive machine session optimized for development.

    Once created, sessions can be accessed via VSCode, Jupyter-lab, or SSH interfaces.
    Grid manages the installation of any/all core libraries, drivers, and interfaces
    to the outside world. Sessions can be run on anything from a small 2 CPU core +
    4GB memory instance to a monster machine with 96 CPU cores + 824 GB memory + eight
    V100 GPUs + 40 GBPS netowrk bandwidth (no, those values aren't typos!); or really
    anything in between.
    """
    client = Grid()
    client.check_is_blocked()

    # make a fun random name when user does not pass in a name
    if name is None:
        name = unique_name()

    #  If the user has not passed a grid config file,
    #  then generate one with a set of default options.
    #  We'll add a default instance and the user's
    #  default credentials.
    if not config:
        _grid_config = _generate_default_interactive_config(
            datastore_name=datastore_name,
            datastore_version=datastore_version,
            datastore_mount_dir=datastore_mount_dir,
            cluster=cluster,
            instance_type=instance_type,
            disk_size=disk_size,
            use_spot=use_spot
        )

    else:
        _grid_config = config

    validate_config(cfg=_grid_config)
    validate_and_configure_datastore(grid_config=_grid_config)

    #  Send to client.
    client.create_interactive_node(config=_grid_config, name=name, description=description)


@session.command()
@rich_click.argument('interactive_node', type=str, nargs=1)
def pause(interactive_node: str) -> None:
    """Pauses a session identified by the INTERACTIVE_NODE name.

    Pausing a session stops the running instance (and any computations being
    performed on it - be sure to save your work!) and and billing of your account
    for the machine. The session can be resumed at a later point with all your
    persisted files and saved work unchanged.
    """
    client = Grid()
    client.pause_interactive_node(interactive_node_name=interactive_node)


@session.command()
@rich_click.argument('interactive_node', type=str, nargs=1)
def resume(interactive_node: str) -> None:
    """Resumes a session identified by INTERACTIVE_NODE."""
    client = Grid()
    client.resume_interactive_node(interactive_node_name=interactive_node)


@session.command()
@rich_click.argument('interactive_node', type=str, nargs=1)
def delete(interactive_node: str) -> None:
    """Deletes a session identified by INTERACTIVE_NODE.

    Deleting a session will stop the running instance (and any computations being
    performed on it) and billing of your account. All work done on the machine
    is permenantly removed, including all/any saved files, code, or downloaded
    data (assuming the source of the data was not a grid datastore - datastore
    data is not deleted).
    """
    client = Grid()
    client.delete_interactive_node(interactive_node_name=interactive_node)


@session.command()
@rich_click.argument('interactive_node', type=str, nargs=1, help='Name of the session to change')
@rich_click.argument('instance_type', type=str, nargs=1, help='Instance type to change to')
@click.option(
    '--use_spot', type=bool, default=None, show_default=True, help='Use a spot instance to launch the session'
)
def change_instance_type(interactive_node: str, instance_type: str, use_spot: Optional[bool]) -> None:
    client = Grid()
    client.change_interactive_node_type(
        interactive_node_name=interactive_node, instance_type=instance_type, use_spot=use_spot
    )


def _update_ssh_config_and_check_ixsession_status(ctx, param, value: str) -> int:
    """
    This updates the SSH config for interactive nodes and
    also checks if those nodes can be interacted with SSH
    before attempting an operation. This prevents SSHing into
    nodes that are not un a running state, e.g. paused or pending.

    This manages a section within user's ssh config file
    for all interactive nodes shh config details

    Afterwards you can use systems ssh & related utilities
    (sshfs, rsync, ansible, whatever) with interactive nodes
    directly

    The default file is ~/.ssh/config and  can be changed via
    envvar GRID_SSH_CONFIG

    Returns
    --------
    value: str
        Unmodified value if valid
    """
    client = Grid()
    nodes = client.sync_ssh_config()

    click.echo(f"Sync config for {len(nodes)} interactive nodes")

    target_ixsession = None
    for node in nodes:
        if node["name"] == value:
            target_ixsession = node

    # Check if interactive session exists at all
    if not target_ixsession:
        session_names = [n["name"] for n in nodes]
        raise click.BadArgumentUsage(
            f"Interactive session {value} does not exist. "
            f"Available Interactive Sessions are: {', '.join(session_names)}"
        )

    # Check if the node is in 'running' status
    if target_ixsession["status"] != "running":
        running_ixsessions = [n["name"] for n in nodes if n["status"] == "running"]
        raise click.BadArgumentUsage(
            f"Interactive session {value} is not ready. "
            f"Sessions that are ready to SSH are: {', '.join(running_ixsessions)}"
        )

    return value


@session.command()
@rich_click.argument(
    'node_name',
    type=str,
    callback=_update_ssh_config_and_check_ixsession_status,
    help="The name of the node. This command executes ssh <node name>.",
)
@rich_click.argument('ssh_args', nargs=-1, type=click.UNPROCESSED, help="Arguments to be forwarded to the SSH command.")
def ssh(node_name, ssh_args):
    """SSH into the interactive node identified by NODE_NAME.

    If you'd like the full power of ssh, you can use any ssh client and
    do `ssh <node_name>`. This command is stripped down version of it.

    Example:

        1. Path to custom key:

        grid session ssh satisfied-rabbit-962 -- -i ~/.ssh/my-key

        2. Custom ssh option:

        grid session ssh satisfied-rabbit-962 -- -o "StrictHostKeyChecking accept-new"
    """
    subprocess.check_call(['ssh', node_name, *ssh_args])


@session.command()
@rich_click.argument('interactive_node', type=str, nargs=1, callback=_update_ssh_config_and_check_ixsession_status)
@rich_click.argument('mount_dir', type=str, nargs=1)
def mount(interactive_node, mount_dir):
    r"""Mount session directory to local. The session is identified by
    INTERACTIVE_NODE and MOUNT_DIR is a path to a directory on the local machine.

    To mount a filesystem use:
    ixNode:[dir] mountpoint

    Examples:
        # Mounts the home directory on the interactive node in dir data
        grid interactive mount bluberry-122 ./data

        # mounts ~/data directory on the interactive node to ./data
        grid interactive mount bluberry-122:~/data ./data

    To unmount it:
      fusermount3 -u mountpoint   # Linux
      umount mountpoint           # OS X, FreeBSD

    Under the hood this is just passing data to sshfs after syncing grid's interactive,
    i.e. this command is dumbed down sshfs

    See Also:
        grid sync-ssh-config --help
    """
    if ':' not in interactive_node:
        interactive_node += ":/home/jovyan"

    client = Grid()
    client.sync_ssh_config()

    try:
        subprocess.check_call(['sshfs', interactive_node, mount_dir])
    except FileNotFoundError:
        raise click.ClickException('Unable to mount: sshfs was not found')
    except subprocess.CalledProcessError as e:
        raise click.ClickException(f'Unable to mount: sshfs failed with code {e.returncode}')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/auth_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name='grid/v1/auth_service.proto',
    package='grid.v1',
    syntax='proto3',
    serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
    create_key=_descriptor._internal_create_key,
    serialized_pb=
    b'\n\x1agrid/v1/auth_service.proto\x12\x07grid.v1\x1a\x1cgoogle/api/annotations.proto\"_\n\x0cLoginRequest\x12\x1a\n\x08username\x18\x01 \x01(\tR\x08username\x12\x17\n\x07\x61pi_key\x18\x02 \x01(\tR\x06\x61piKey\x12\x1a\n\x08\x64uration\x18\x03 \x01(\x03R\x08\x64uration\"%\n\rLoginResponse\x12\x14\n\x05token\x18\x01 \x01(\tR\x05token\",\n\x0eRefreshRequest\x12\x1a\n\x08\x64uration\x18\x01 \x01(\x03R\x08\x64uration\"\'\n\x0fRefreshResponse\x12\x14\n\x05token\x18\x01 \x01(\tR\x05token\"\x0f\n\rLogoutRequest\"\x10\n\x0eLogoutResponse2\x92\x02\n\x0b\x41uthService\x12Q\n\x05Login\x12\x15.grid.v1.LoginRequest\x1a\x16.grid.v1.LoginResponse\"\x19\x82\xd3\xe4\x93\x02\x13\"\x0e/v1/auth/login:\x01*\x12Y\n\x07Refresh\x12\x17.grid.v1.RefreshRequest\x1a\x18.grid.v1.RefreshResponse\"\x1b\x82\xd3\xe4\x93\x02\x15\"\x10/v1/auth/refresh:\x01*\x12U\n\x06Logout\x12\x16.grid.v1.LogoutRequest\x1a\x17.grid.v1.LogoutResponse\"\x1a\x82\xd3\xe4\x93\x02\x14\"\x0f/v1/auth/logout:\x01*B2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3',
    dependencies=[
        google_dot_api_dot_annotations__pb2.DESCRIPTOR,
    ]
)

_LOGINREQUEST = _descriptor.Descriptor(
    name='LoginRequest',
    full_name='grid.v1.LoginRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='username',
            full_name='grid.v1.LoginRequest.username',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='username',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='api_key',
            full_name='grid.v1.LoginRequest.api_key',
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='apiKey',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
        _descriptor.FieldDescriptor(
            name='duration',
            full_name='grid.v1.LoginRequest.duration',
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='duration',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=69,
    serialized_end=164,
)

_LOGINRESPONSE = _descriptor.Descriptor(
    name='LoginResponse',
    full_name='grid.v1.LoginResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='token',
            full_name='grid.v1.LoginResponse.token',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='token',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=166,
    serialized_end=203,
)

_REFRESHREQUEST = _descriptor.Descriptor(
    name='RefreshRequest',
    full_name='grid.v1.RefreshRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='duration',
            full_name='grid.v1.RefreshRequest.duration',
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='duration',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=205,
    serialized_end=249,
)

_REFRESHRESPONSE = _descriptor.Descriptor(
    name='RefreshResponse',
    full_name='grid.v1.RefreshResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name='token',
            full_name='grid.v1.RefreshResponse.token',
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode('utf-8'),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            json_name='token',
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=251,
    serialized_end=290,
)

_LOGOUTREQUEST = _descriptor.Descriptor(
    name='LogoutRequest',
    full_name='grid.v1.LogoutRequest',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=292,
    serialized_end=307,
)

_LOGOUTRESPONSE = _descriptor.Descriptor(
    name='LogoutResponse',
    full_name='grid.v1.LogoutResponse',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax='proto3',
    extension_ranges=[],
    oneofs=[],
    serialized_start=309,
    serialized_end=325,
)

DESCRIPTOR.message_types_by_name['LoginRequest'] = _LOGINREQUEST
DESCRIPTOR.message_types_by_name['LoginResponse'] = _LOGINRESPONSE
DESCRIPTOR.message_types_by_name['RefreshRequest'] = _REFRESHREQUEST
DESCRIPTOR.message_types_by_name['RefreshResponse'] = _REFRESHRESPONSE
DESCRIPTOR.message_types_by_name['LogoutRequest'] = _LOGOUTREQUEST
DESCRIPTOR.message_types_by_name['LogoutResponse'] = _LOGOUTRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LoginRequest = _reflection.GeneratedProtocolMessageType(
    'LoginRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LOGINREQUEST,
        '__module__': 'grid.v1.auth_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.LoginRequest)
    }
)
_sym_db.RegisterMessage(LoginRequest)

LoginResponse = _reflection.GeneratedProtocolMessageType(
    'LoginResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LOGINRESPONSE,
        '__module__': 'grid.v1.auth_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.LoginResponse)
    }
)
_sym_db.RegisterMessage(LoginResponse)

RefreshRequest = _reflection.GeneratedProtocolMessageType(
    'RefreshRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _REFRESHREQUEST,
        '__module__': 'grid.v1.auth_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.RefreshRequest)
    }
)
_sym_db.RegisterMessage(RefreshRequest)

RefreshResponse = _reflection.GeneratedProtocolMessageType(
    'RefreshResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _REFRESHRESPONSE,
        '__module__': 'grid.v1.auth_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.RefreshResponse)
    }
)
_sym_db.RegisterMessage(RefreshResponse)

LogoutRequest = _reflection.GeneratedProtocolMessageType(
    'LogoutRequest',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LOGOUTREQUEST,
        '__module__': 'grid.v1.auth_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.LogoutRequest)
    }
)
_sym_db.RegisterMessage(LogoutRequest)

LogoutResponse = _reflection.GeneratedProtocolMessageType(
    'LogoutResponse',
    (_message.Message, ),
    {
        'DESCRIPTOR': _LOGOUTRESPONSE,
        '__module__': 'grid.v1.auth_service_pb2'
        # @@protoc_insertion_point(class_scope:grid.v1.LogoutResponse)
    }
)
_sym_db.RegisterMessage(LogoutResponse)

DESCRIPTOR._options = None

_AUTHSERVICE = _descriptor.ServiceDescriptor(
    name='AuthService',
    full_name='grid.v1.AuthService',
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=328,
    serialized_end=602,
    methods=[
        _descriptor.MethodDescriptor(
            name='Login',
            full_name='grid.v1.AuthService.Login',
            index=0,
            containing_service=None,
            input_type=_LOGINREQUEST,
            output_type=_LOGINRESPONSE,
            serialized_options=b'\202\323\344\223\002\023\"\016/v1/auth/login:\001*',
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='Refresh',
            full_name='grid.v1.AuthService.Refresh',
            index=1,
            containing_service=None,
            input_type=_REFRESHREQUEST,
            output_type=_REFRESHRESPONSE,
            serialized_options=b'\202\323\344\223\002\025\"\020/v1/auth/refresh:\001*',
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name='Logout',
            full_name='grid.v1.AuthService.Logout',
            index=2,
            containing_service=None,
            input_type=_LOGOUTREQUEST,
            output_type=_LOGOUTRESPONSE,
            serialized_options=b'\202\323\344\223\002\024\"\017/v1/auth/logout:\001*',
            create_key=_descriptor._internal_create_key,
        ),
    ]
)
_sym_db.RegisterServiceDescriptor(_AUTHSERVICE)

DESCRIPTOR.services_by_name['AuthService'] = _AUTHSERVICE

# @@protoc_insertion_point(module_scope)



from shlex import split
from typing import Optional

import click
from gql import Client, gql
from rich.table import Table

from grid.cli.observables.base import BaseObservable
from grid.cli.utilities import get_experiment_duration_string, get_experiment_queued_duration_string, get_param_values


class Experiment(BaseObservable):
    def __init__(self, client: Client, identifier: str):
        self.client = client
        self.run_name = identifier

        super().__init__(client=client)

    # This doesn't seem to be called
    def get_history(self, experiment_id: Optional[str] = None, is_global: Optional[bool] = False):
        """
        Parameters
        ----------
        experiment_id: Optional[str]
            Experiment ID
        is_global: Optional[bool], default False
            Returns status of session from everyone in the team
        """
        self.spinner.start()
        self.spinner.text = 'Getting Experiments ...'

        query = gql(
            """
        query (
            $runName: ID!
        ) {
            getExperiments(runName: $runName) {
                name
                status
                invocationCommands
                createdAt
                finishedAt
                commitSha
                run {
                    runId
                }
                startedRunningAt
            }
        }
        """
        )
        params = {'runName': self.run_name}

        result = self.client.execute(query, variable_values=params)
        if not result['getExperiments']:
            click.echo(f'No experiments available for run "{self.run_name}"')
            return

        self.spinner.text = 'Done!'
        self.spinner.ok("✔")
        self.spinner.stop()
        table = self.render_experiments(result['getExperiments'])
        self.console.print(table)

    def get(self, is_global: Optional[bool] = False):
        """
        Parameters
        ----------
        is_global: Optional[bool], default False
            if True, returns status of experiment(s) of the everyone in the team (TODO)
        """
        with self.spinner:
            self.spinner.text = 'Fetching experiment status ...'

            # User can pass runs as username:experiment_name to fetch other users runs
            username = None
            run_name = self.run_name
            split = self.run_name.split(":")
            if len(split) > 2:
                raise ValueError(f"Error while parsing {self.run_name}. Use the format <username>:<experiment-name>")
            elif len(split) == 2:
                username = split[0]
                run_name = split[1]

            query = gql(
                """
            query (
                $runName: ID!, $username: String
            ) {
                getExperiments(runName: $runName, username: $username) {
                    name
                    status
                    invocationCommands
                    createdAt
                    finishedAt
                    commitSha
                    run {
                        runId
                    }
                    startedRunningAt
                }
            }
            """
            )
            params = {'runName': run_name, "username": username}
            result = self.client.execute(query, variable_values=params)

            experiments = result['getExperiments']
            if not experiments:
                click.echo(f'No experiments available for run "{self.run_name}"')
                return

        table = self.render_experiments(experiments)
        self.console.print(table)
        return result

    @staticmethod
    def render_experiments(experiments) -> Table:
        base_columns = [
            'Experiment',
            'Command',
            'Status',
            'Queued Duration',
            'Run Duration',
        ]
        if not experiments:
            return BaseObservable.create_table(columns=base_columns)

        command = experiments[0]['invocationCommands']
        toks = split(command)
        hparams = [tok.replace('--', '') for tok in toks if '--' in tok]

        table_columns = base_columns + hparams
        table = BaseObservable.create_table(columns=table_columns)

        for experiment in experiments:
            # Split hparam vals
            command = experiment['invocationCommands']
            base_command, *hparam_vals = get_param_values(command)

            duration_str = get_experiment_duration_string(
                created_at=experiment['createdAt'],
                started_running_at=experiment['startedRunningAt'],
                finished_at=experiment['finishedAt']
            )
            queued_duration_str = get_experiment_queued_duration_string(
                created_at=experiment['createdAt'],
                started_running_at=experiment['startedRunningAt'],
            )
            table.add_row(
                experiment['name'], base_command, experiment['status'], queued_duration_str, duration_str, *hparam_vals
            )
        return table

    def follow(self, is_global: Optional[bool] = False):
        pass



import base64
from collections import OrderedDict
from inspect import signature
from pathlib import Path
import shlex
from typing import Any, Dict, List, Optional, Union
import warnings
import webbrowser

import yaml

import grid.cli.globals as env
import grid.sdk._gql.queries
from grid.sdk._gql.queries import check_github_repo_accessible, repository_presigned_url, start_run_mutation
from grid.sdk.datastore import list_all_available_datastores
from grid.sdk.source_code.dependency_manager import check_dependency_listing, find_package_manager
from grid.sdk.source_code.git import (
    _check_if_remote_head_is_different,
    _check_if_uncommited_files,
    add_git_root_path,
    check_github_repository,
    execute_git_command,
)
from grid.sdk.source_code.repository import LocalRepository
from grid.sdk.utils.name_generator import unique_name
from grid.sdk.utils.navigation import set_directory


def _check_run_name_is_valid(value: str):
    """Click callback that checks if a Run contains reserved names."""
    if value is not None:
        fail = False

        #  Check if the input is alphanumeric.
        _run_name = value.replace('-', '')
        if not _run_name.isalnum():
            fail = True

        #  Check if the allowed `-` character is not used
        #  at the end of the string.
        elif value.endswith('-') or value.startswith('-'):
            fail = True

        #  Check that the run name does not contain any
        #  uppercase characters.
        elif any(x.isupper() for x in value):
            fail = True

        if fail:
            raise ValueError(
                f"Invalid Run name: {value} the Run name must be lower case "
                "alphanumeric characters or '-', start with an alphabetic "
                "character, and end with an alphanumeric character (e.g. 'my-name', "
                " or 'abc-123')."
            )

    return value


def _generate_default_grid_config(
    instance_type: str,
    cluster: Optional[str] = None,
    use_spot: Optional[bool] = None,
    grid_strategy: Optional[str] = None,
    grid_trials: Optional[int] = None,
    gpus: Optional[int] = None,
    memory: str = None,
    cpus: Optional[int] = None,
    datastore_name: Optional[str] = None,
    datastore_version: Optional[int] = None,
    datastore_mount_dir: Optional[str] = None,
    framework: Optional[str] = None,
    localdir: Optional[bool] = None,
    dockerfile: Optional[bool] = None,
    scratch_size: Optional[int] = None,
    scratch_mount_path: Optional[str] = None,
    dependency_file: Optional[str] = None
):
    """
    Generates a new default config file for user if user hasn't
    submitted one.
    """

    provider = {
        'vendor': "aws",
        'credentials': "n/a",
        'region': "us-east-1",
    }
    if cluster is not None:
        provider['cluster'] = cluster

    _grid_config = {
        'compute': {
            'provider': provider,
            'train': {
                'framework': framework,
                'datastore_name': datastore_name,
                'datastore_version': datastore_version,
                'datastore_mount_dir': datastore_mount_dir,
                'instance': instance_type,
                'use_spot': use_spot or False,
                'nodes': 0,  # How many nodes are created at first.
                'disk_size': 200,
                'gpus': gpus,
                'memory': memory,
                'cpus': cpus,
                'scratch': [],
                'localdir': localdir,
                'dockerfile': dockerfile,
            }
        }
    }

    if dependency_file:
        package_manager = find_package_manager(dependency_file)
        if not localdir:
            dependency_file = add_git_root_path(dependency_file)

        _grid_config['compute']['train']['dependency_file_info'] = {
            'package_manager': package_manager,
            'path': dependency_file
        }

    if scratch_size:
        _grid_config['compute']['train']['scratch'].append({'size_gb': scratch_size, 'mount_path': scratch_mount_path})

    #  We add a `hyper_params` key to the default config
    #  if the user has passed the `grid_strategy` argument.
    if grid_strategy:
        if not _grid_config.get('hyper_params'):
            _grid_config['hyper_params'] = {}

        _grid_config['hyper_params'] = {'settings': {'strategy': grid_strategy}, 'params': {}}

        #  We also add the key trials if that has been
        #  passed. trials is only valid when doing a `random_search`
        #  at the moment. Our parser will not be bothered by its
        #  presence if we pass it by default.
        if grid_trials:
            _grid_config['hyper_params']['settings']['trials'] = grid_trials

    return _grid_config


def _verify_entrypoint_script_is_valid(p: Path):
    """Verify if the entrypoint script is a file extension that grid can actually execute.
    """
    if not p.exists() or not p.is_file():
        raise FileNotFoundError(f"Entrypoint script file {p} does not exist.")

    if p.suffix not in ['.py', '.sh', '.jl']:
        raise ValueError(
            f'We only support Python (.py), shell (.sh), or Julia (.jl) scripts. Got: {p}.'
            f'Email support@grid.ai if you want to request support for this file type.'
        )


def _aws_node_to_nickname():
    aws_node_to_nicknames = OrderedDict({
        'p3.16xlarge': '8_v100_16gb',
        'p3dn.24xlarge': '8_v100_32gb',
        'g4dn.metal': '8_t4_16gb',
        'p2.8xlarge': '8_k80_12gb',
        'p3.8xlarge': '4_v100_16gb',
        'g4dn.12xlarge': '4_t4_16gb',
        'g3.16xlarge': '4_m60_8gb',
        'g3.8xlarge': '2_m60_8gb',
        'p3.2xlarge': '1_v100_16gb',
        # 'p4d.24xlarge': '8_a100_40gb',  # currently not supported
        'g4dn.8xlarge': '1_t4_16gb',
        'g4dn.4xlarge': '1_t4_16gb',
        'g4dn.2xlarge': '1_t4_16gb',
        'g4dn.xlarge': '1_t4_16gb',
        'g4dn.16xlarge': '1_t4_16gb',
        'p2.xlarge': '1_k80_12gb',
        'g3s.xlarge': '1_m60_8gb',
        'g3.4xlarge': '1_m60_8gb',
        't2.large': '2_cpu_8gb',
        't2.medium': '2_cpu_4gb'
    })
    return aws_node_to_nicknames


def _nickname_to_aws_nodes():
    aws_node_to_nickname = _aws_node_to_nickname()
    aws_nickname_to_node = {v: k for k, v in aws_node_to_nickname.items()}
    return aws_nickname_to_node


def _resolve_instance_type_nickname(value):
    """
    Enables instance type shortcuts like:
    2_cpu_4gb for t2.large
    """
    nickname = value.lower()

    aws_nickname_to_node = _nickname_to_aws_nodes()

    # validate potential options for the node name
    possible_values = list(aws_nickname_to_node.keys()) + list(aws_nickname_to_node.values())
    if nickname not in possible_values:
        possible_options = '\n'.join(list(aws_nickname_to_node.keys()))
        ValueError(f'{nickname} is not an available instance_type\n try one of these:\n{possible_options}')

    instance_type = nickname

    # if the value has _ then it's a nickname
    if '_' in nickname:
        instance_type = aws_nickname_to_node[nickname]
    return instance_type


def _get_instance_types(ctx, args, incomplete):
    # TODO: these should be retrieved from backend
    return list(_aws_node_to_nickname().keys())


def _parse_script_args(args):
    """Small state machine to parse the script name and script args __before__ the click parser is called."""
    flags = ['--', '--ignore_warnings']

    # states: 'parameter_name', 'parameter_value', 'flag', 'script_name', 'script_arg'
    state = None

    click_args = []
    script_name = None
    script_args = []

    for arg in args:
        arg = str(arg)
        if state in ['script_name', 'script_arg']:
            script_args.append(arg)
        else:
            if arg.startswith('--'):
                if arg in flags:
                    state = 'flag'
                else:
                    state = 'parameter_name'
            elif state == 'parameter_name':
                state = 'parameter_value'
            else:
                state = 'script_name'
                script_name = arg

            if state != 'script_name':
                click_args.append(arg)
    return click_args, script_name, script_args


def read_config(path: Union[str, Path]) -> dict:
    """
    Parameters
    ----------
    path:
        path to file on disk
    """
    if not isinstance(path, Path):
        path = Path(path)
    if not path.is_file():
        raise FileNotFoundError(f'grid config file {path} does not exist')

    #  Loads the YML file as passed by the user.
    try:
        with open(path) as f:
            grid_config = yaml.safe_load(f.read())
        if not isinstance(grid_config, dict):
            raise Exception("Unexpected file structure")
    except Exception as e:
        raise ValueError(f'Could not load your YAML config file: {e}')

    #  Adds required structure to the base YML file, if that structure isn't there.
    if 'compute' not in grid_config:
        grid_config['compute'] = {}
    if 'train' not in grid_config['compute']:
        grid_config['compute']['train'] = {}
    return grid_config


def check_description_isnt_too_long(value: str):
    """Click callback that checks if the description isn't too long."""
    if value is not None and len(value) > 200:
        raise ValueError('Description should have at most ' f'200 characters, yours has {len(value)}.')
    return value


def validate_and_configure_datastore(grid_config: dict):
    """
    Validate the existence of provided datastore and user's access. Inject datastore id to the
    config, based on the name and version provided. If version is not provided, this function also
    injects the maximum version to the config
    """
    try:
        datastore_name = grid_config["compute"]["train"]["datastore_name"]
        datastore_version = grid_config["compute"]["train"]["datastore_version"]
        if isinstance(datastore_version, int):
            datastore_version = str(datastore_version)
    except:
        return

    if not datastore_name:
        return

    split = datastore_name.split(":")
    if len(split) > 2:
        raise ValueError(f"Error while parsing {datastore_name}. Use the format <username>:<datastore-name>")
    elif len(split) == 2:
        datastore_name = split[1]
        datastores = list_all_available_datastores(include_teams=True)
    else:
        datastores = list_all_available_datastores()

    usable_version = -1
    datastore_id = None
    for ds in sorted(datastores, key=lambda x: x.version):
        if datastore_name == ds.name and ds.snapshot_status == "succeeded":
            current_version = int(ds.version)
            if datastore_version and datastore_version == str(current_version):
                datastore_id = ds.datastore_id
                usable_version = datastore_version
                break
            usable_version = max(usable_version, current_version)
            datastore_id = ds.datastore_id

    if not datastore_id:
        raise ValueError(f'No ready-to-use datastore found with name {datastore_name} and version {datastore_version}')

    if datastore_version and datastore_version != str(usable_version):
        raise ValueError(f"Unable to list datastores with name {datastore_name} and version {datastore_version}")

    if not datastore_version:
        warnings.warn(
            f'No --grid_datastore_version passed for datastore: {datastore_name}. '
            f'Using version: {usable_version}'
        )

    # inject valid details back to config
    grid_config["compute"]["train"]["datastore_id"] = datastore_id
    grid_config["compute"]["train"]["datastore_name"] = datastore_name
    grid_config["compute"]["train"]["datastore_version"] = str(usable_version)

    return grid_config


def start_run(
    entrypoint: Path,
    script_args: str = '',
    strategy: str = 'grid_search',
    trials: int = None,
    framework: str = 'lightning',
    dependency_file: Optional[str] = None,
    instance_type: str = 't2.medium',
    use_spot: bool = False,
    cpus: int = 0,
    gpus: int = 0,
    memory: Optional[str] = None,
    datastore_name: Optional[str] = None,
    datastore_version: Optional[str] = None,
    datastore_mount_dir: Optional[str] = None,
    scratch_size: Optional[int] = None,
    scratch_mount_path: str = '/tmp/scratch',
    name: Optional[str] = None,
    description: Optional[str] = None,
    cluster: Optional[str] = None,
    localdir: bool = False,
    dockerfile: Optional[str] = None,
    config: Optional[Union[str, Path]] = None
) -> str:
    """Runs scripts in remote infrastructure.

    Returns
    -------
    str
        run name which was started
    """

    _invocation_args = {}
    sig = signature(start_run)
    for p_name, param in sig.parameters.items():
        if p_name == ['entrypoint', 'script_args']:
            continue
        if locals()[p_name] != param.default:
            _invocation_args[p_name] = locals()[p_name]

    if config is not None:
        #  Adds required structure to the base YML file, if that structure isn't there.
        config = read_config(config)
    else:
        config = {}

    if name is not None:
        name = _check_run_name_is_valid(name)
    else:
        name = unique_name()

    if strategy not in ['grid_search', 'random_search']:
        raise ValueError(f"strategy {strategy} is not valid.")

    check_description_isnt_too_long(description)
    instance_type = _resolve_instance_type_nickname(instance_type)

    if framework not in ['lightning', 'torch', 'tensorflow', 'julia', 'torchelastic']:
        raise ValueError(framework)

    _verify_entrypoint_script_is_valid(Path(entrypoint))
    script_arguments = shlex.split(script_args)

    shlex_g_args = ' '.join([shlex.quote(f'--{k}') + ' ' + shlex.quote(f'{v}') for k, v in _invocation_args.items()])
    shlex_entrypoint = shlex.quote(f'{entrypoint}')
    shlex_script_args = ' '.join([shlex.quote(x) for x in shlex.split(script_args)])
    invocation_command = shlex.quote(f'grid run {shlex_g_args} {shlex_entrypoint} {shlex_script_args}')

    # make a fun random name when user does not pass in a name
    name = unique_name() if name is None else name

    #  If the user has not passed a grid config file, then generate one with a set
    #  of default options. We'll add a default instance and the user's default credentials.
    if not config:
        _grid_config = _generate_default_grid_config(
            instance_type=instance_type,
            cluster=cluster,
            use_spot=use_spot,
            grid_strategy=strategy,
            grid_trials=trials,
            gpus=gpus,
            memory=memory,
            cpus=cpus,
            datastore_name=datastore_name,
            datastore_version=datastore_version,
            datastore_mount_dir=datastore_mount_dir,
            framework=framework,
            localdir=localdir,
            dockerfile=dockerfile,
            scratch_size=scratch_size,
            scratch_mount_path=scratch_mount_path,
            dependency_file=dependency_file
        )

    else:
        _grid_config = config
        _grid_config['compute'].setdefault('train', {})
        _grid_config['compute']['train'].setdefault('localdir', localdir)
        _grid_config['compute']['train'].setdefault('dockerfile', dockerfile)

    validate_and_configure_datastore(grid_config=_grid_config)

    # Warn user if they have not specified a cpu or gpu number so cpus will be set to 1
    cpus = _grid_config['compute']['train'].get('cpus')
    gpus = _grid_config['compute']['train'].get('gpus')
    if not cpus and not gpus:
        _grid_config['compute']['train']['cpus'] = 1

    #  Send to client.
    return train(
        config=_grid_config,
        run_name=name,
        run_description=description,
        entrypoint=entrypoint,
        script_args=script_arguments,
        invocation_command=invocation_command,
        localdir=localdir,
        dependency_file=dependency_file,
    )


def train(
    config: Dict[str, Any],
    run_name: str,
    run_description: str,
    entrypoint: Path,
    script_args: List[str],
    invocation_command: str,
    localdir: bool = False,
    dependency_file=None
) -> str:
    """
    Submits a Run to backend from a local script.

    Parameters
    ----------
    config: dict
        YAML config
    run_name: str
        Run name
    run_description: str
        Run description.
    entrypoint: str
        Entrypoint script.
    script_args: List[str]
        Script arguments passed from command line.
    invocation_command: str
    localdir: bool
        If True, trigger the local directory run workflow
    dependency_file: str
        Dependency file path. Defaults to None
    """
    #  Base64-encode the config object either passed
    #  or constructed.
    config_encoded = base64.b64encode(yaml.dump(config).encode('ascii')).decode('ascii')
    env.logger.debug(config_encoded)

    # we will be using `github_repository` and `commit_sha` as a repository ID
    # and checksum version temporarily; this will change when we call `createRun`
    # directly

    if localdir:
        # Checking dependencies and suggesting when needed
        repository_path = Path(entrypoint).parent.absolute()
        with set_directory(repository_path):
            check_dependency_listing(repo_root=repository_path, dependency_file=dependency_file)

            _entrypoint = entrypoint

            # get path from entrypoint
            repository = LocalRepository(path=repository_path)

            cluster_id = config["compute"]["provider"].get("cluster")
            presigned_url = repository_presigned_url(package_name=f"{repository.version}.tar.gz", cluster_id=cluster_id)
            # hack to avoid collision between spinner and rich progress
            repository.upload(url=presigned_url)
            github_repository = repository_path
            commit_sha = repository.version

    # only apply git checks in case this is a git run
    else:
        any_repo_path = Path(entrypoint).parent
        with set_directory(any_repo_path):
            # Checking dependencies and suggesting when needed
            check_dependency_listing(repo_root=any_repo_path, dependency_file=dependency_file)

            if not env.GRID_SKIP_GITHUB_TOKEN_CHECK:
                grid.sdk._gql.queries._check_user_github_token()

            check_github_repository()

            # Check if repository contains uncommitted files.
            _check_if_uncommited_files()

            # Check if remote is in sync with local.
            _check_if_remote_head_is_different()

            #  Get commit SHA
            commit_sha = execute_git_command(['rev-parse', 'HEAD'])

            #  Get repo name
            github_repository = execute_git_command(["config", "--get", "remote.origin.url"])

            #  Clean up the repo name
            github_repository = github_repository.replace('git@github.com:', 'github.com/')
            github_repository = github_repository.replace('.git', '')

            #  Add the root path to the entrypoint script.
            _entrypoint = add_git_root_path(entrypoint)
            _entrypoint = Path(entrypoint).relative_to(any_repo_path)

            if not check_github_repo_accessible(github_repository):
                settings_url = env.GRID_URL.replace("graphql", "") + "#/settings?tabId=integrations"
                webbrowser.open(settings_url, new=2, autoraise=True)
                raise PermissionError(
                    f'Grid cannot access {github_repository}. Please grant access on the settings page: {settings_url}'
                )

    start_run_mutation(
        script_args=script_args,
        entrypoint=str(_entrypoint),
        config_encoded=config_encoded,
        run_name=run_name,
        run_description=run_description,
        commit_sha=commit_sha,
        github_repository=github_repository,
        invocation_command=invocation_command
    )

    return run_name
